#grp-filters .grp-autocomplete-wrapper-fk {
    display: block;
    width: 100%;
}

#grp-filters .grp-autocomplete-wrapper-fk .ui-autocomplete-input {
    padding-right: 25px;
}

#grp-filters .grp-autocomplete-wrapper-fk .grp-related-remove {
    padding: 0;
    margin: 0 -24px 0 0;

    -moz-border-radius-topright: 3px;
    -webkit-border-top-right-radius: 3px;
    border-top-right-radius: 3px;
    -moz-border-radius-bottomright: 3px;
    -webkit-border-bottom-right-radius: 3px;
    border-bottom-right-radius: 3px;
}

#grp-filters .grp-autocomplete-wrapper-fk .grp-loader {
    right: 0;

    -moz-border-radius-topright: 3px;
    -webkit-border-top-right-radius: 3px;
    border-top-right-radius: 3px;
    -moz-border-radius-bottomright: 3px;
    -webkit-border-bottom-right-radius: 3px;
    border-bottom-right-radius: 3px;
}

.ui-menu li.ui-menu-item a.ui-state-focus {
    color: #fff;
    border: 1px solid #333;
    background: #444;
}
